### Clear memory, set working directory and load unit root test and lag selection functions

rm(list=ls(all=T));

setwd("../"); # need to set working directory
source("./R_functions/phillips_sadf_gsadf.R");
source("./R_functions/seq_lag.R");

##################################################################################
#### Baumeister and Kilian (BK) and Futures Data Sets ############################
#### Consensus Economics data are proprietary and are not reported ###############
#### Column 1: Date for the Spot Price 
#### Column 2: Date for Future/Expected Spot Price, 1 month ahead
#### Column 3: Date for Future/Expected Spot Price, 3 months ahead
#### Column 4: Date for Future/Expected Spot Price, 12 months ahead
#### Column 5: Spot Price from Bloomberg
#### Column 6: BK/Futures 3m forecast
#### Column 7: BK/Futures 12m forecast
#### Column 8: Future Spot Price, 3 months ahead 
#### Column 9: Future Spot Price, 12 months ahead 
#### Column 10: 3 month Forecast Error (3m Future Spot Price - 3m BK/Futures Forecast)
#### Column 11: 12 month Forecast Error (12m Future Spot Price - 12m BK/Futures Forecast)
#### Column 12: Future Spot Price, 1 month ahead 
#### Column 13: BK/Futures 1m forecast  
##########################################################################################################

spotData=read.table("./Data/spot_data.txt",header=T,sep="\t");
futData=read.table("./Data/fut_data.txt",header=T,sep="\t");
bkData=read.table("./Data/bk_data.txt",header=T,sep="\t");

########### Selecting the minimum window size
T=nrow(futData);
minws=floor(T*(0.01+1.8/sqrt(T))); 

###### BSADF Critical Values 

bsadf_crit=read.table("./Critical_values/bsadf_t276_critvalues95.txt",header=T,sep="\t"); 
bsadf_crit[bsadf_crit==-99999]=NA;

######################### Levels ##############################

# ADF, SADF, and GSADF statistics for the Spot, HW/Futures 3m forecast (Column 6), HW/Futures 12m forecast (Column 7)

unr_results=matrix(,5,3); 
colnames(unr_results)=c("ADF","SADF","GSADF");
rownames(unr_results)=c("Spot","BK3m","Fut3m","BK12m","Fut12m");

##### Data matrix: Spot, BK +3, Fut +3, BK +12, Fut +12
aux_matrix=cbind(spotData[,2],bkData[,6],futData[,6],bkData[,7],futData[,7]);


p=1; #lag specification

for(i in 1:5){
  yy=as.vector(aux_matrix[,i],"numeric");
  aux=rls.gsadf(yy,lags=p,wmin=minws);
  unr_results[i,]=c(aux$adf,aux$sadf,aux$gsadf);
  if(i==1){
    bsadf_stat=c(rep(-999,p+1),aux$bsadf); # -999 for original obs lost due to lags
  }else{
    bsadf_stat=cbind(bsadf_stat,c(rep(-999,p+1),aux$bsadf));
  }
  
}

colnames(bsadf_stat)=c("Spot","BK3m","Fut3m","BK12m","Fut12m");


print(paste("Unit Root Test Results for Spot, Futures, and Expectations"));
print(unr_results);


############ Plotting Spot and 3m Futures/Expectations ########################

par(ask=T);
par(cex=1);
par(mar=c(5.1,4.1,4.1,1.1));
date_graph=vector();
for(i in 1:T){
  aux=strsplit(as.vector(spotData[i,1]),"/");
  if(nchar(aux[[1]][2])==2){
    date_graph[i]=paste(aux[[1]][2],aux[[1]][3],sep="-");
  }else{
    date_graph[i]=paste(aux[[1]][1],paste(0,aux[[1]][2],sep=""),sep="-");
  }
}

bsadf_stat[bsadf_stat==-999]=NA;

plot.ts(bsadf_stat[,1],ylim=c(-5,6),xlab=NULL,ylab="BSADF",axes=F,lwd=2);

axis(1,las=2,labels = date_graph[seq(1,T,by=15)],at=seq(1,T,by=15));

for(i in 2:3){
  lines(bsadf_stat[,i],lty=1,lwd=2,col=i);
}
lines(bsadf_crit[,1],lty=2,lwd=2,col="black");

legend(y=6.3,x=9, c(expression(S[t]), expression({E[t]^F} (S[t+3])),expression(F[t]^3),"95% CV"), col = c(1:3,1),
       text.col = "black", lty = c(rep(1,3),2),lwd=2, merge = TRUE, bg = NULL, bty="n",y.intersp=1.2,cex=.9);
box();
axis(2,las=1);
################################ Plotting 12m Futures/Expectations ######################################

plot.ts(bsadf_stat[,4],ylim=c(-5,6),xlab=NULL,ylab="BSADF",axes=F,lwd=2,col=1); # same colors for same expectation/fut series 

axis(2,las=1);
axis(1,las=2,labels = date_graph[seq(1,T,by=15)],at=seq(1,T,by=15));

lines(bsadf_stat[,5],lty=1,lwd=2,col=2);
lines(bsadf_crit[,1],lty=2,lwd=2,col="black");

legend(y=6.2,x=9, c(expression({E[t]^F} (S[t+12])),expression(F[t]^12),"95% CV"), col = c(1,2,1),
       text.col = "black", lty = c(rep(1,2),2),lwd=2, merge = TRUE, bg = NULL, bty="n",y.intersp=1.2,cex=.9);

box();

##################################################################################
############# Future Spot- BK Expectation - Seq Lag Selection ####################

# ADF, SADF, and GSADF statistics for the Future Spot - Consensus/HW/Futures 3m forecast (Column 10), and Future Spot - BK 12m forecast (Column 11)

unr_results_ferrors=matrix(,3,3);
colnames(unr_results_ferrors)=c("ADF","SADF","GSADF");
rownames(unr_results_ferrors)=c("S3m-BK3m","S12m-BK12m","BK3m-BK12m");

maxlag=11;
diff_IC=matrix(,3,1); # six series 
colnames(diff_IC)="Seq";
rownames(diff_IC)=c("S3m-BK3m","S12m-BK12m","BK3m-BK12m");

##### Data matrix: Future Spot - BK +3; Future Spot - BK +12
aux_matrix=cbind(bkData[,10],bkData[,11]);

for(i in 1:2){
  yy=as.vector(aux_matrix[,i],"numeric");
  diff_IC[i,]=ar_seq(x=diff(yy),mp=maxlag);
  aux=rls.gsadf(yy,lags=diff_IC[i],wmin=minws);
  unr_results_ferrors[i,]=c(aux$adf,aux$sadf,aux$gsadf);
}

# Data matrix: BK +3 - BK +12

yy=cbind(bkData[-c(1:9),6]-bkData[c(1:(T-9)),7]);
diff_IC[3]=ar_seq(x=diff(yy),mp=maxlag);
aux=rls.gsadf(yy,lags=diff_IC[3],wmin=minws);
unr_results_ferrors[3,]=c(aux$adf,aux$sadf,aux$gsadf);


print(paste("Unit Root Test Results for Future Spot - Expectations"));
print(unr_results_ferrors);
